<?php
/*
** Server List PHP script by Cataphract v2
** Based on the Jazz2 Add-on for mIRC
** Lisboa, 24 de Maro de 2004
** By Cataphract (Cataphract@netcabo.pt)
** The author disclaims copyright to this source code
*****************************************************
*/
// This file gets the players and level of a jazz2 server.

//This script is to be included in another PHP script.
//You must define the variables $si_ip and $si_version before
// including this script.
//$si_ip is the IP of the Jazz2 server. $si_version should be '4'
// if the server is a 1.24 server, '1' if the server is a 1.23
// server. The variables can be passed through the URL ip= and
//version=.
//You cannot pass one of the variables through the URL and
//the other by setting it directly. You must choose one of the
//paths.
//Please see 'example.php' for more information.

/*
** VARIABLES SET:
** Array $si_players - array of player objects
 List of methods for the above class:
 getname([TRUE/FALSE])
   if TRUE, 's and |'s will be stripped
 getchar()
 getnumber()
 getcolors([fourBytes/bytesArray/spaceSepDec/decArray/hexArray])
   gets the fur colors (return value according to argument given)
** $si_number_players - number of jazzers playing in that server
** $si_errno - error number (0 if none)
** $si_errstr - error message (empty string if nome)
*********************************************************
*/

/*--------------------------------------------------------------*/
/*                    Script configuration                      */
/*--------------------------------------------------------------*/

//Change this variable to the name you want for the
//pseudo-player that will join the server. BEWARE: very long
//names crash the servers!
$si_conf['botname'] = "Ƞ    PHP";

//Error reporting level (You might want to change this for
//debugging).
error_reporting (0);

/* Don't touch anything below this comment unless you know what
you are doing */

//Class definition
class SiPlayer {
	var $name;
	var $char;
	var $number;
	var $colors; //Stored as a four characters long string
	
	//Constructor
	function SiPlayer($name,$char,$number,$colors) {
		$this->name = $name;
		$this->char = $char;
		$this->number = $number;
		$this->colors = $colors;
	}
	
	//Methods
	function getname($strip=FALSE) {
		return $strip?si_remlixo($this->name):$this->name;
	}
	function getchar() { return $this->char; }
	function getnumber() { return $this->number; }
	function getcolors($mode="spaceSepDec") {
		/* Read modes: spaceSepDec, spaceSepHex, fourBytes
		bytesArray, decArray, hexArray */
		$t = array();
		for ($i=0; $i<4; $i++) {
			$t[$i] = substr($this->colors,$i,1);
		}
		if ($mode=="fourBytes") return $this->colors;;
		if ($mode=="bytesArray") return $t;
		elseif ($mode=="spaceSepDec") {
			$out = '';
			for ($i=0; $i<4; $i++) { $out .= " ".ord($t[$i]); }
			return substr($out,1);
		}
		elseif ($mode=="spaceSepHex") {
			$out = '';
			for ($i=0; $i<4; $i++) $out .= " ".dechex(ord($t[$i]));
			return substr($out,1);
		}
		elseif ($mode=="decArray") {
			for ($i=0; $i<4; $i++) $t[$i] = ord($t[$i]);
			return $t;
		}
		elseif ($mode=="hexArray") {
			for ($i=0; $i<4; $i++) $t[$i] = dechex(ord($t[$i]));
			return $t;
		}
		else {
			return $this->getcolors();
		}
	}
}

//Auxiliar function
//Strips the |'s and  blocks:
function si_remlixo($v) {
	while (($n = strpos($v,"")) !== FALSE) { 
		$v = substr($v,0,$n).substr($v,$n +2);
	}
	$v = str_replace("|","",$v);
	return $v;
}

//Required input variables handling
if ((!isset($si_ip)) || (!isset($si_version))) {
	if ((!isset($_GET['ip'])) || (!isset($_GET['version'])))
		die("Insufficient parameters in si.php");
  $si_ip = $_GET['ip'];
  $si_version = $_GET['version'];
}

//Returns an array of objects of class SiPlayer:
function si_anPl($data) {
	global $si_conf;
	$res = array(); // Initializes result variable
	$r = 2; // Offset of character being read
	while (TRUE) {
		/* In each iteration a player is read */
		$part = "skip"; //What we are reading
		$num = ''; //Player slot number
		$char = ''; //Player's character
		$name = ''; //Player's name
		$fur = ''; //Player's fur
		$byte = ''; //Byte being read
		$furnum = 0; //Number of fur color (1,2,3 or 4)
		
		while ($r++ < strlen($data) -1) {
			/* In each iteration a byte is read */
			$byte = substr($data,$r,1);
			if ($part == "skip") $part = "num";
			
			elseif ($part == "num") { //Reading player's slot number
				$num = ord($byte);
				$part = "char";
			}
			
			elseif ($part == "char") { //Reading player's character
				//Arrays for replacement rules
				$inByte = array(
					"\x01","\x00","\x02","\x10","\x11","\x12",
					"\x0A","\x0B","\xF1","\xF0","\x13","\xF2",
					"\x13"
				);
				$outChar = array(
					"Spaz","Jazz","Lori","Jazz","Spaz","Lori",
					"Jazz","Spaz","Spaz","Jazz","Bird","Lori",
					"Bird"
				);
				$char = str_replace($inByte,$outChar,$byte);
				$part = "fur";
			}
			
			elseif ($part == "fur") { //Reading player's fur
				$furnum++;
				$fur .= $byte;
				if ($furnum == 4) {
					$part = "name";
				}
			}
			
			elseif ($part == "name") { //Reading player's name
				if ($byte == "\x00") {
					//In this case, the player reading has ended
					if ($name != $si_conf['botname']) {
						array_push($res,
							new SiPlayer(
								$name,
								$char,
								$num,
								$fur
							)
						);
					}
					continue 2; //Starts an external iteration
				}
				$name = $name.$byte;
			}
        } // End of internal cycle
	break; //Breaks if internal cycle ends (no continue is called)
	} // End of external cycle
	return $res;
} // End of function si_anPl()

function si_main($address,
	$port,
	&$target,
	&$levelname,
	&$errno,
	&$errstr
) {
	global $si_version,$si_ip,$si_conf;
	
	//Checks whether the host is resolved
	if (($ips = gethostbynamel($address)) !== FALSE)
		$address = $ips[0];
	else {
		$errno = -3;
		$errstr = "Could not resolve server hostname.";
		return;
	}
	//Opens the socket (timeout 5s):
	$socket = fsockopen ($address,$port,$errno,$errstr,5);
	if ($socket === FALSE) return;
	//Sets 5s timeout for reading/writing:
	stream_set_timeout($socket,5);
	
	$rcData = '';
	$toSend = "\x09\x0F\x01\x04\x32".$si_version."\x20\x20\x01";
	$stage = 1; // Initialize packet count
	
	// Read cycle
	$i = 0; // Just in case...
	while($i++<25) {
		
		//Send pending data
		if (fwrite($socket,$toSend,strlen($toSend)) === FALSE) {
			$errno = -1;
			$errstr = "Error while writing to socket.";
			return;
		}
		//Read packet
		$rcData = fread($socket,8192);
		
		if ($stage == 1) {
			//Get levelname and information for sending number & name
			$levelname = substr($rcData,5,strlen($rcData) -15);
			$playernum = substr($rcData,3,1); //Slot number
			$toSend = "\x01\x0E\x01".$playernum.
				"\x01\x00\x00\x00\x00".$si_conf['botname'];
		}
		elseif ($stage == 2) {
			if (substr($rcData,-1) != "\x00") {
				//In case the last character is not \x00 we have an
				// unexpected response.
				$errno = -2;
				$errstr = "Unexpected response from server.";
				return;
			}
			$target =
				si_anPl($rcData);
			break; // I've gotten all I wanted from the server
		}
		$stage++;
	} // End of cycle
fclose ($socket);
} // End of function si_main()

//Initialize variables
$si_players = array();
$si_errno = 0;
$si_errstr = '';
$si_levelname = '';

// Calls the function defined above:
si_main($si_ip,10052,$si_players,$si_levelname,$si_errno,$si_errstr);

//Number of players:
$si_number_players = count($si_players);

?>