<?php
/*
** Server List PHP script by Cataphract v2
** Based on the Jazz2 Add-on for mIRC
** Lisboa, 24 de Maro de 2004
** By Cataphract (Cataphract@netcabo.pt)
** The author disclaims copyright to this source code
*****************************************************
*/
// This file gets the server list.

//This script is to be included in another PHP script.
//Please see 'example.php' for an example on how to do this

/*
** VARIABLES SET:
** Array $sl_servers - array of server objects
 List of methods for the above class:
 getname([TRUE/FALSE])
  if TRUE, 's and |'s will be stripped
 getip()
 gettype()
 getversion([TRUE/FALSE])
  if TRUE replaces 1.21 with 1.23
 getgamemode()
 getplayers([TRUE/FALSE])
  if TRUE, shows "players/capacity"
 getcapacity()
 getuptime([TRUE/FALSE])
  if TRUE shows number os seconds
** $sl_servers_number - number of listed servers
** $sl_total_players - number of jazzers playing
** $sl_total_capacity - capacity of all servers
** $sl_errno - socket error number (if any)
** $sl_errno - socket error number (0 if none)
** $sl_errstr - socket error message (empty string if nome)
*********************************************************
*/

/*--------------------------------------------------------------*/
/*                    Script configuration                      */
/*--------------------------------------------------------------*/

//Use this variable to set the hostname of the list server.
//You may want to define it externally
if (!isset($sl_conf['listserver'])) $sl_conf['listserver'] = 'jazz.nimrod-online.com';

//Error reporting level (You might want to change this for
//debugging).
error_reporting (0);

/* Don't touch anything below this comment unless you know what
you are doing */

//Class definition
class SlServer {
	var $name;
	var $ip;
	var $type;
	var $version;
	var $gamemode;
	var $players;
	var $capacity;
	var $uptime;
	
	//Constructor
	function SlServer(
		$name,
		$ip,
		$type,
		$version,
		$gamemode,
		$players,
		$capacity,
		$uptime
	) {
		$this->name = $name;
		$this->ip = $ip;
		$this->type = $type;
		$this->version = $version;
		$this->gamemode = $gamemode;
		$this->players = $players;
		$this->capacity = $capacity;
		$this->uptime = $uptime;
	}
	
	//Methods
	function getname($strip=FALSE) {
		return $strip?sl_remlixo($this->name):$this->name;
	}
	function getip() { return $this->ip; }
	function gettype() { return $this->type; }
	function getversion($assumelatest=FALSE) {
		$v = $this->version;
		if ($assumelatest) $v = str_replace("1.21","1.23",$v);
		return $v;
	}
	function getgamemode() { return $this->gamemode; }
	function getplayers($alsocapacity=FALSE) {
		return $alsocapacity?$this->players."/".$this->capacity:
			$this->players;
	}
	function getcapacity() { return $this->capacity; }
	function getuptime($raw=FALSE) {
		return $raw?$this->uptime:sl_conv($this->uptime);
	}
}

//Auxiliar functions
function sl_conv($time) {
	if ($time < 60) return $time."s";
	elseif ($time < 3600) return floor($time / 60)."m"." ".
		($time % 60)."s";
	else return floor($time / 3600)."h"." ".floor(($time % 3600)/60).
		"m"." ".(($time % 3600)%60)."s";
}
//Strips the |'s and 's together with the following character:
function sl_remlixo($v) {
	while (($n = strpos($v,"")) !== FALSE) { 
		$v = substr($v,0,$n).substr($v,$n +2);
	}
	$v = str_replace("|","",$v);
	return $v;
}

/* Initializes de variables concerning socket error control
** (strictly for clarity) */
$sl_errno = 0;
$sl_errstr = '';

// Reads the server list from the server and returns the result
function sl_readSL($listserver,
	$service_port,
	&$errno,
	&$errstr
) {
	//Checks whether the host is resolved
	if (($ips = gethostbynamel($listserver)) !== FALSE)
		$listserver = $ips[0];
	else {
		$errno = -3;
		$errstr = "Could not resolve list server hostname.";
		return '';
	}
	// Create a TCP/IP socket
	$socket = fsockopen($listserver,$service_port,$errno,$errstr,5);
	if ($socket === FALSE) return '';
	
	$rcData = ''; //Received data accumulated here
	while (!feof($socket)) {
		$rcData .= fread($socket, 8192);
	}
	fclose($socket);
	return $rcData;
}

// Uses the socket handling function defined abovve
$sl_data = sl_readSL($sl_conf['listserver'],
	10057,
	$sl_errno,
	$sl_errstr
);

$sl_servers = array(); // Target array initialization
// Initializes the remaining variables:
$sl_total_players = 0;
$sl_total_capacity = 0;
$sl_servers_number = 0;

// Loads the data into the appropriate array
function sl_loadData($data,
	&$targetarray,
	&$total_players,
	&$total_capacity,
	&$servers_number
) {
	$servers = split("\n",$data);
	$servers_number = count($servers) - 1;
	$r = -1;
	while (++$r < $servers_number) {
		if (!isset($servers[$r])) continue;		
		$i = split(" ",str_replace("  "," ",$servers[$r]),9);
		
		list($ip,$port) = split(":",$i[0]);
		// $i[1] says whether the server is being mirrored. Irrelevant.
		$type = $i[2];
		$gamemode = $i[3];
		$version = $i[4].$i[5];
		$uptime = $i[6];
		list($players,$capacity) =
			split("/",str_replace(array("[","]"),"",$i[7]));
		$servername = htmlspecialchars(trim($i[8],"\n\r"));
		
		$total_capacity = $total_capacity + $capacity;
		$total_players = $total_players + $players;
		
		array_push($targetarray,
			new SlServer($servername,
				$ip,
				$type,
				$version,
				$gamemode,
				$players,
				$capacity,
				$uptime
			)
		);
	}
}

// Calls the function above:
sl_loadData($sl_data,
	$sl_servers,
	$sl_total_players,
	$sl_total_capacity,
	$sl_servers_number
);

?>
